@@ -50,6 +50,15 @@ high_contrast_changed_cb (AdwSettingsImplGSettings *self)
   adw_settings_impl_set_high_contrast (ADW_SETTINGS_IMPL (self), high_contrast);
 }
 
+static void
+accent_color_changed_cb (AdwSettingsImplGSettings *self)
+{
+  AdwAccentColor accent_color =
+    g_settings_get_enum (self->interface_settings, "accent-color");
+
+  adw_settings_impl_set_accent_color (ADW_SETTINGS_IMPL (self), accent_color);
+}
+
 static void
 adw_settings_impl_gsettings_dispose (GObject *object)
 {
@@ -76,12 +85,14 @@ adw_settings_impl_gsettings_init (AdwSettingsImplGSettings *self)
 
 AdwSettingsImpl *
 adw_settings_impl_gsettings_new (gboolean enable_color_scheme,
-                                 gboolean enable_high_contrast)
+                                 gboolean enable_high_contrast,
+                                 gboolean enable_accent_colors)
 {
   AdwSettingsImplGSettings *self = g_object_new (ADW_TYPE_SETTINGS_IMPL_GSETTINGS, NULL);
   GSettingsSchemaSource *source;
   gboolean found_color_scheme = FALSE;
   gboolean found_high_contrast = FALSE;
+  gboolean found_accent_colors = FALSE;
 
   /* While we can access gsettings in flatpak, we can't do anything useful with
    * them as they aren't propagated from the system. */
@@ -90,20 +101,34 @@ adw_settings_impl_gsettings_new (gboolean enable_color_scheme,
 
   source = g_settings_schema_source_get_default ();
 
-  if (enable_color_scheme && adw_get_disable_portal ()) {
+  if ((enable_color_scheme || enable_accent_colors) && adw_get_disable_portal ()) {
     GSettingsSchema *schema =
       g_settings_schema_source_lookup (source, "org.gnome.desktop.interface", TRUE);
 
-    if (schema && g_settings_schema_has_key (schema, "color-scheme")) {
-      found_color_scheme = TRUE;
+    if (schema) {
       self->interface_settings = g_settings_new ("org.gnome.desktop.interface");
 
-      color_scheme_changed_cb (self);
+      if (g_settings_schema_has_key (schema, "color-scheme")) {
+        found_color_scheme = TRUE;
 
-      g_signal_connect_swapped (self->interface_settings,
-                                "changed::color-scheme",
-                                G_CALLBACK (color_scheme_changed_cb),
-                                self);
+        color_scheme_changed_cb (self);
+
+        g_signal_connect_swapped (self->interface_settings,
+                                  "changed::color-scheme",
+                                  G_CALLBACK (color_scheme_changed_cb),
+                                  self);
+      }
+
+      if (g_settings_schema_has_key (schema, "accent-color")) {
+        found_accent_colors = TRUE;
+
+        accent_color_changed_cb (self);
+
+        g_signal_connect_swapped (self->interface_settings,
+                                  "changed::accent-color",
+                                  G_CALLBACK (accent_color_changed_cb),
+                                  self);
+      }
     }
 
     g_clear_pointer (&schema, g_settings_schema_unref);
@@ -130,7 +155,8 @@ adw_settings_impl_gsettings_new (gboolean enable_color_scheme,
 
   adw_settings_impl_set_features (ADW_SETTINGS_IMPL (self),
                                   found_color_scheme,
-                                  found_high_contrast);
+                                  found_high_contrast,
+                                  found_accent_colors);
 
   return ADW_SETTINGS_IMPL (self);
 }
